#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <iostream>

using namespace std;

struct CE_Info {
    int numUtilities;
    int numParameters;
    int numObservables;
};

struct Utility {
    char name[50];
    char units[50];
    char goal[50];
   /* string name;
    string units;
    string goal;*/
    float target;
};

struct Affect {
    Utility u; 
    //string relation;
    char relation[50];
};      

struct Parameter {
    /*string name;
    string units;*/
    char name[50];
    char units[50];
    float min;
    int numAffects; 
    Affect affection_list[10];
    float max;
    float step;
};      
        
struct Observable {
    //string name;
    char name[50];
    Affect affection_list[10];
    int numAffects;
};

// error handling
void error(char *msg)
{
    perror(msg);
    exit(0);
}

void ReadMessage(int socket,char * buffer) {
    int i,n;

    n = recv(socket,buffer,256,MSG_PEEK);
    for(i=0;i<256;i++){
	    if(strcmp(&buffer[i],"\0") == 0) break;
    }
    n = recv(socket,buffer,i+1,0);
    if (n < 0) 
         error("ERROR reading from socket");
//    printf("ReadMessage:%s %d\n",buffer,n);
}

// main client socket
int main(int argc, char *argv[])
{
    int sockfd, portno, n;
    struct sockaddr_in serv_addr;
    struct hostent *server;

    // create a socket
    char buffer[256];
    char buffer1[256];
    if (argc < 3) {
       fprintf(stderr,"usage %s hostname port\n", argv[0]);
       exit(0);
    }
    portno = atoi(argv[2]);
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0) 
        error("ERROR opening socket");
    server = gethostbyname(argv[1]);
    if (server == NULL) {
        fprintf(stderr,"ERROR, no such host\n");
        exit(0);
    }
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr, 
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length);
    serv_addr.sin_port = htons(portno);
    // cast sockaddr_in to sockaddr
    if (connect(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr)) < 0) 
        error("ERROR connecting");
    
    int i, j;
    Utility uList[10];
    Parameter pList[10];
    Observable oList[10];
    CE_Info ce_info;

    // read unitilities
    // numUtilities
    bzero(buffer,256);
    ReadMessage(sockfd,buffer);
    ce_info.numUtilities = atoi(buffer);
    printf("number of utilities: %d\n", ce_info.numUtilities);
        
    for (i = 0; i < ce_info.numUtilities; i++){
        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
	    printf("Name: %s\n", buffer);
        strcpy(uList[i].name, buffer);
                 
        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
	    printf("Units: %s\n", buffer);
        strcpy(uList[i].units, buffer);

        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
        printf("Goal: %s\n", buffer);
        strcpy(uList[i].goal, buffer);
        
	    bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
        printf("Target: %s\n", buffer);
        uList[i].target = atof(buffer);
    }

    // read parameters
    // numParameters
    bzero(buffer,256);
    ReadMessage(sockfd,buffer);
    ce_info.numParameters = atoi(buffer);
    printf("number of parameters: %d\n", ce_info.numParameters);
        
    for (i = 0; i < ce_info.numParameters; i++){
        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
	    printf("Name: %s\n", buffer);
        strcpy(pList[i].name, buffer);
                 
        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
	    printf("Units: %s\n", buffer);
        strcpy(pList[i].units, buffer);

        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
        printf("Min: %s\n", buffer);
        pList[i].min = atof(buffer);
        
        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
        printf("Max: %s\n", buffer);
        pList[i].max = atof(buffer);
        
	    bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
        printf("Step: %s\n", buffer);
        pList[i].step = atof(buffer);
    
        // numAffects
        bzero(buffer,256);
        ReadMessage(sockfd,buffer);
        pList[i].numAffects = atoi(buffer);
        printf("number of affects: %d\n", pList[i].numAffects);
        
        for (j = 0; j < pList[i].numAffects; j++){
            bzero(buffer,256);
	        ReadMessage(sockfd,buffer);
	        printf("Utility name: %s\n", buffer);
            strcpy(pList[i].affection_list[j].u.name, buffer);
                 
            bzero(buffer,256);
	        ReadMessage(sockfd,buffer);
	        printf("Relation: %s\n", buffer);
            strcpy(pList[i].affection_list[j].relation, buffer);
        }

    }
    
    // read obervables
    // numParameters
    bzero(buffer,256);
    ReadMessage(sockfd,buffer);
    ce_info.numObservables = atoi(buffer);
    printf("number of observables: %d\n", ce_info.numObservables);
        
    for (i = 0; i < ce_info.numObservables; i++){
        bzero(buffer,256);
	    ReadMessage(sockfd,buffer);
	    printf("Name: %s\n", buffer);
        strcpy(pList[i].name, buffer);
                 
        // numAffects
        bzero(buffer,256);
        ReadMessage(sockfd,buffer);
        oList[i].numAffects = atoi(buffer);
        printf("number of affects: %d\n", oList[i].numAffects);
        
        for (j = 0; j < oList[i].numAffects; j++){
            bzero(buffer,256);
	        ReadMessage(sockfd,buffer);
	        printf("Utility name: %s\n", buffer);
            strcpy(oList[i].affection_list[j].u.name, buffer);
                 
            bzero(buffer,256);
	        ReadMessage(sockfd,buffer);
	        printf("Relation: %s\n", buffer);
            strcpy(oList[i].affection_list[j].relation, buffer);
        }
    }

    
    printf("xml transmission done\n");


  return 0;
}
